
\\ Les fonctions suivantes se trouvent dans le logiciel cmh (j'ai fait quelques légères modifications)
\\ voir http://cmh.gforge.inria.fr/

/* Contient:
-invariants_streng_to_igusaclebsch(JJJ);
-invariants_igusaclebcsch_to_clebsch(III);
-algo de Mestre: curve_from_igusaclebsch_on_C(III);
*/

invariants_streng_to_igusaclebsch(JJJ)=
{/* Streng defines: j1, j2, j3 as:
  * I4I6'/I10, I2I4^2/I10, I4^5/I10^2
  * where I6' = (I2I4-3I6)/2
  */
 my(j1,j2,j3,i5,i2,i3,J2,J4,J6,J10,I2,I4,I6,I10);

 [j1,j2,j3]=JJJ;
 /* These are the [i5,i2,i3] used by Weng */
 i5=(j2-2*j1)/3;    /* I4I6/I10 */
 i2=j2^3/j3;        /* I2^3I4/I10 */        \\ i5=0 => i3=0 => non déf
 i3=i5*i2/j2;       /* I2^2I6/I10 */

 /* These are the Igusa ones */
 J2 = 1;
 J10= 8*i5/(i2*i3);
 J4 = (-16*i5 + i3)/(24*i3);
 J6 = (80*i5*i2 - 384*i5*i3 + i2*i3)/(432*i2*i3);

 /* Now the Igusa-Clebsch ones */
 I2 = 8*J2;
 I4 = 4*(J2^2 - 24*J4);
 I6 = 8*(J2*(J2^2 - 20*J4) - 72*J6);
 I10= 4096*J10;

return([I2,I4,I6,I10]);
}


invariants_igusaclebcsch_to_clebsch(III)=
{my(I2,I4,I6,I10,A,B,C,D);
 [I2,I4,I6,I10]=III;
 A =-I2/120 ;
 B =(I4+720*A^2)/6750 ;
 C =(I6-8640*A^3+108000*A*B)/202500 ;
 D =(I10+62208*A^5-972000*A^3*B-1620000*A^2*C+3037500*A*B^2+6075000*B*C)/(-4556250) ;

return([A,B,C,D]);
}




/**************** DEBUT algo de Mestre *************/

curve_from_igusaclebsch_on_C(III)=
{my(I2,I4,I6,I10,LM,L,M,P,u,v,Ltm,a,b,vM,x,x1,x2,x3,m,t,hf,g,z);
 [I2,I4,I6,I10]=III; 
 x='x;
 x1='x1;
 x2='x2;
 x3='x3;
 m='m;
 t='t;
 g=automorphism_group_info(III);

\\ if(g!=2,print("g="g));

\\    if(g!=2,warning("curve_from_igusaclebsch untested beyond the case of trivial automorphism groups"););
\\   if(g==240,return([0,x^5-x]);); characteristic 5 only 
 if(g==48,return(30*x^6 + 79*x^5 + 80*x^4 + 40*x^3 + 10*x^2 + x));       \\return([0,x^5-x]);
 if(g==10,return(31*x^6 + 80*x^5 + 80*x^4 + 40*x^3 + 10*x^2 + x));       \\return([0,x^5-1]););
 if(g==24,return(x^6-1));
 if(g==8,
      z=(3*(36000*I10 + I2^3*I4 + 44*I2*I4^2 + 6*I2^2*I6 - 480*I4*I6))/ (10800000*I10 + 9*I2^5 + 700*I2^3*I4 - 12400*I2*I4^2 - 3600*I2^2*I6 + 48000*I4*I6);
      return((2*z + 40)*x^6 + (z + 92)*x^5 + 86*x^4 + 41*x^3 + 10*x^2 + x));   \\return([0,x^5+x^3+z*x]);
 if(g==12,
      z = (2*I2*I4 - 20*I6)/(3*I2^3 + 140*I2*I4 - 800*I6);
      return(x^6 + x^3 + z));
 if(g==4,return(cardona_curve_from_igusaclebsch(III)));
 
 LM=mestre_conic_and_cubic(III);
 L=LM[1];
 M=LM[2];
 P=find_point_on_conic_on_C(L);
 u=P[1];
 v=P[2];
 Ltm=subst(subst(subst(L,x1,u+m*t),x2,v+t),x3,1);
 b=polcoeff(Ltm,1,t);
 a=polcoeff(Ltm,2,t);
 vM=subst(subst(subst(M,x1,a*u-b*m),x2,a*v-b),x3,a);
 hf = subst(vM,m,x);

return(hf);
}

automorphism_group_info(III)=
{my(I2,I4,I6,I10,i1,i2,i3,d41,d42,d61,d62,R2,precs);
 [I2,I4,I6,I10]=III; 
 precs=round(default(realprecision,)*3/4);

 if(( abs(I2)<10^-precs&&abs(I4)<10^-precs&&abs(I6)<10^-precs ),return(10););                \\ correction ([I2,I4,I6]==[0,0,0]),return(10););
 
 i1 = I2^5/I10;
 i2 = I2^3*I4/I10;
 i3 = I2^2*I6/I10;
 if(abs(i1-51200000/3)<10^-precs && abs(i2-480000)<10^-precs && abs(i3-148000)<10^-precs, return(24));     \\ correction if((i123==[51200000/3,480000,148000]), return(24););
 if(abs(i1-400000)<10^-precs && abs(i2+20000)<10^-precs && abs(i3+2000)<10^-precs, return(48));   \\ correction if(i123==[400000, -20000, -2000], return(48););
 
 d41=72000*I10 + I2^3*I4 - 82*I2*I4^2 - 3*I2^2*I6 + 240*I4*I6;
 d42=-270000*I10*I2 + 3*I2^4*I4 + 734*I2^2*I4^2 + 640*I4^3 - 9*I2^3*I6 - 4620*I2*I4*I6 + 7200*I6^2;
 if (abs(d41)<10^-precs && abs(d42)<10^-precs, return(8));   \\ correction if ([d41,d42] == [0,0], return(8););
 
 d61=108000*I10 + I2^3*I4 - 208*I2*I4^2 - 12*I2^2*I6 + 960*I4*I6;
 d62=5400*I10*I2 - 13*I2^2*I4^2 + 4*I4^3 + 96*I2*I4*I6 - 180*I6^2;
 if (abs(d61)<10^-precs && abs(d62)<10^-precs, return(12));   \\ correction if ([d61,d62] == [0,0], return(12););
 
 R2 = 125971200000*I10^3+236196*I10^2*I2^5+19245600*I10^2*I2^3*I4- 507384000*I10^2*I2*I4^2-972*I10*I2^6*I4^2-77436*I10*I2^4*I4^3+ 592272*I10*I2^2*I4^4+I2^7*I4^4-41472*I10*I4^5+78*I2^5*I4^5- 159*I2^3*I4^6+80*I2*I4^7-104976000*I10^2*I2^2*I6+ 2099520000*I10^2*I4*I6+5832*I10*I2^5*I4*I6+870912*I10*I2^3*I4^2*I6- 4743360*I10*I2*I4^3*I6-12*I2^6*I4^3*I6-1332*I2^4*I4^4*I6+ 1728*I2^2*I4^5*I6-384*I4^6*I6-8748*I10*I2^4*I6^2- 3090960*I10*I2^2*I4*I6^2+9331200*I10*I4^2*I6^2+54*I2^5*I4^2*I6^2+ 8910*I2^3*I4^3*I6^2-6048*I2*I4^4*I6^2+3499200*I10*I2*I6^3- 108*I2^4*I4*I6^3-29376*I2^2*I4^2*I6^3+6912*I4^3*I6^3+81*I2^3*I6^4+ 47952*I2*I4*I6^4-31104*I6^5;
 if(abs(R2)<10^-precs, return(4));  \\ correction if(R2==0, return(4););
 
return(2);
}

cardona_curve_from_igusaclebsch(III)=
{my(A,B,C,D,A11,A12,A22,A33,a111,a112,a122,a133,a222,a233,P1,P2,P3,f,x,precs);
 [A,B,C,D]=invariants_igusaclebcsch_to_clebsch(III);
 x='x;

 precs=round(default(realprecision,)*3/4); 

 A11 = 2*C+A*B/3;
 A12 = 2*B^2/3+2*A*C/3;
 A22 = D;
 A33 = C*D-2/9*B^4+1/6*A*B*D-4/9*A*B^2*C-2/9*A^2*C^2;

 a111 = 4/75*D-8/225*B*C+4/675*A^2*C;
 a112 = (4*B^3)/675 + (8*A*B*C)/675 + (8*C^2)/225 + (2*A*D)/225;
 a122 = (2*A*B^3)/675 + (8*A^2*B*C)/2025 + (8*B^2*C)/675 + (4*A*C^2)/225 + (2*B*D)/225;
 a133 = -(A^2*B^4)/2025 + (8*B^5)/2025 - (4*A^3*B^2*C)/6075 + (14*A*B^3*C)/2025 + (2*A^2*B*C^2)/2025 + (8*B^2*C^2)/675 + (4*A*C^3)/675 + (A*B^2*D)/225 + (2*A^2*C*D)/675 + (2*B*C*D)/225 - (2*D^2)/75;
 a222 = (2*B^4)/225 + (4*A*B^2*C)/225 + (16*A^2*C^2)/2025 + (4*B*C^2)/675 - (2*C*D)/225;
 a233 = (A*B^5)/2025 + (2*A^2*B^3*C)/1215 - (2*B^4*C)/2025 + (8*A^3*B*C^2)/6075 + (2*A*B^2*C^2)/2025 + (8*A^2*C^3)/2025 - (4*B*C^3)/675 + (2*B^3*D)/675 + (A*B*C*D)/675 - (2*C^2*D)/225 - (A*D^2)/225;

 if(abs(A22)>10^-precs,         \\correction A22!=0,
    P1 = -2*A12-2*A22*x;
    P2 = A11-A22*x^2;
    P3 = A11+2*A12*x+A22*x^2;
    f = -A33*a111*P1^3-3*A33*a112*P1^2*P2-3*A33*a122*P1*P2^2+ 3*A22*a133*P1*P3^2-A33*a222*P2^3+3*A22*a233*P2*P3^2;
   ,
    P1 = -A11*x^2;
    P2 = -2*(A12 + A11*x);
    P3 = x*(2*A12 + A11*x);
    f = -(a111*A33*P1^3) - 3*a112*A33*P1^2*P2 - 3*a122*A33*P1*P2^2 - a222*A33*P2^3 + 3*A11*a133*P1*P3^2 + 3*A11*a233*P2*P3^2;
  );

return(f);
}

mestre_conic_and_cubic(III)=
{my(DP,A,B,C,D,U, A11, A22, A33, A23, A31, A12,  C11, C22, C33, C23, C31, C12, C13, a111, a112, a113, a122, a123, a133, a222, a223, a233, a333, c111, c112, c113, c122, c123, c133, c222, c223, c233, c333,L,M,P,x1,x2,x3,precs);
 [A,B,C,D]=invariants_igusaclebcsch_to_clebsch(III);
 DP=III[4];
 x1='x1;
 x2='x2;
 x3='x3;

 precs=round(default(realprecision,)*3/4); 

 if(abs(A)>10^-precs,U=A^6;if(abs(B)>10^-precs,U=B^3;,U=C^2;););              \\ correction A!=0,U=A^6;,if(B!=0,U=B^3;,U=C^2;););
 A11 = 2*C+A*B/3 ;
 A22 = D;
 A33 = B*D/2+2*C*(B^2+A*C)/9 ;
 A23 = B*(B^2+A*C)/3+C*(2*C+A*B/3)/3 ;
 A31 = D;
 A12 = 2*(B^2+A*C)/3 ;
 
 C11 = A11*U^2*DP^8/DP^11 ;
 C22 = A22*DP^10/DP^11 ;
 C33 = A33*U^8/DP^11 ;
 C23 = A23*DP^5*U^4/DP^11 ;
 C31 = A31*DP^4*U^5/DP^11 ;
 C12 = A12*U*DP^9/DP^11 ;
 C13 = C31;  

 a111 = 8*(A^2*C-6*B*C+9*D)/36 ;
 a112 = 4*(2*B^3+4*A*B*C+12*C^2+3*A*D)/36 ;
 a113 = 4*(A*B^3+4*A^2*B*C/3+4*B^2*C+6*A*C^2+3*B*D)/36 ;
 a122 = a113;
 a123 = 2*(2*B^4+4*A*B^2*C+4*A^2*C^2/3+4*B*C^2+3*A*B*D+12*C*D)/36 ;
 a133 = 2*(A*B^4+4*A^2*B^2*C/3+16*B^3*C/3+26*A*B*C^2/3+ 8*C^3+3*B^2*D+2*A*C*D)/36 ;
 a222 = 4*(3*B^4+6*A*B^2*C+8*A^2*C^2/3+2*B*C^2-3*C*D)/36 ;
 a223 = 2*(-2*B^3*C/3-4*A*B*C^2/3-4*C^3+9*B^2*D+8*A*C*D)/36 ;
 a233 = 2*(B^5+2*A*B^3*C+8*A^2*B*C^2/9+2*B^2*C^2/3 -B*C*D+9*D^2)/36 ;
 a333 = 1*(-2*B^4*C-4*A*B^2*C^2-16*A^2*C^3/9-4*B*C^3/3 +9*B^3*D+12*A*B*C*D+20*C^2*D)/36 ;
 
 P = U^(-18)*DP^5 ;

 c111 = a111*P*U^3*DP^12 ;
 c112 = a112*P*U^2*DP^13 ;
 c113 = a113*P*U^6*DP^8 ;
 c122 = a122*P*U*DP^14 ;
 c123 = a123*P*U^5*DP^9 ;
 c133 = a133*P*U^9*DP^4 ;
 c222 = a222*P*DP^15 ;
 c223 = a223*P*U^4*DP^10 ;
 c233 = a233*P*U^8*DP^5 ;
 c333 = a333*P*U^12 ;

 L = C11*x1^2+C22*x2^2+C33*x3^2+2*C12*x1*x2+2*C13*x1*x3+2*C23*x2*x3;
 M = c111*x1^3+c222*x2^3+c333*x3^3+3*c112*x1^2*x2+3*c113*x1^2*x3+ 3*c122*x1*x2^2+3*c133*x1*x3^2+3*c233*x2*x3^2+3*c223*x2^2*x3+ 6*c123*x1*x2*x3;

return([L,M]);
}


find_point_on_conic_on_C(L)=
{my(LL,x2,x3);
 x2='x2;
 x3='x3;
 LL=subst(L,x3,1);
 LL=subst(LL,x2,1);

return([polroots(LL)[1],1,1]);
}


/********* FIN algo de Mestre ************/
/*****************************************/

