
\\ Les fonctions de ce fichier ont été écrites par Pascal Molin (avec quelques légères modifications de ma part)
\\ voir  https://github.com/pascalmolin/hcperiods

\\ La fonction hcInit permet de calculer la matrice des périodes associée à une courbe hyperelliptique E=prod_{i=1}^6(x-ei)
\\ exemple: hcInit(vector(6,i,i))

/******* DEBUT Matrice des périodes  *********/


symplectic_reduction(M) = {
  my(g,N,P,i,j,ii,jj,tmpN,res,u,v,tmpP,tmp);
  g=matsize(M)[1];
  if(g%2,return(0),g/=2);
  N = Vec(M); \\ columns matrices
  P = Vec(matid(2*g));
    for(d = 1, g,
        if(1,
          Mat(vector(2*g,k,if(k<=g,P[2*k-1],P[2*(k-g)])));
          );
  
        /* find a pivot value */
        i = 2*d-1; j = i+1;
        while(!N[j][i],
            print("i",i,"j",j,"->",N[j][i]);
            if(j++>2*g,j=i+++1);
          );
        \\ i and j give a and b
        if(N[j][i]<0,
           P[j] = -P[j];
           N[j] = -N[j];
           for(l=1,2*g,N[l][j] = -N[l][j]);
        );
        ii=i; jj=j;
        while(jj,
        \\print("d=",d," -> i=",i,"&j=",j);
        /* remove all coeffs on column j */
          for(k=2*d-1,2*g, \\ the column
            if(k==jj||!N[k][ii],next()); \\ next if already zero
            if(N[k][ii]%N[jj][ii]==0,
              /* N[jj][ii] divides N[k][ii], set it to zero */
              my(q = N[k][ii]/N[jj][ii]);
              P[k] = P[k]-q*P[jj];
              N[k] = N[k]-q*N[jj];
              for(l=1,2*g,N[l][k] = N[l][k] - q*N[l][jj];);
              ,
              /* otherwise replace N[jj][ii] by the gcd */
              tmp = bezout(N[jj][ii],N[jj][k]);
              u = tmp[1]; v = tmp[2]; d = tmp[3];
              P[ii] = u*P[ii]+v*P[jj];
              N[ii] = u*N[ii]+v*N[jj];
              for(l=1,2*g,N[l][ii] = u*N[l][ii] + v*N[l][jj];);
              \\ now resume on N[jj][ii]
              k--
              );
            );
          if(jj>ii,ii=j;jj=i,jj=0);
  );
  /* and put the 2-2 symplectic at 2*d-1, 2*d */
  tmpN = N[2*d-1]; tmpP = P[2*d-1];
  N[2*d-1] = N[i]; P[2*d-1] = P[i];
  N[i] = tmpN; P[i] = tmpP;
  tmpN = N[2*d]; tmpP = P[2*d];
  N[2*d] = N[j]; P[2*d] = P[j];
  N[j] = tmpN; P[j]= tmpP;
  );
  /* and finally canoniq basis */
  res = Mat(vector(2*g,k,if(k<=g,P[2*k-1],P[2*(k-g)])));
  return(res);
};


intersection_spanning(A,tree)={
  my(n,res,s);
  n = #tree;
  res = matrix(n,n);
  for(i=1,#tree,
    for(j=i+1,#tree,
    s = intersection(A,tree[i][1],tree[i][2],tree[j][1],tree[j][2]);
    res[i,j] = s;
    res[j,i] = -s;
    );
  );
  return(res);
};


intersection(A,ia,ib,ic,id) = {
  if(ia==ib||ic==id,return(0)); \\ bad entry
  if((ia==ic&&ib==id)||(ia==id&&ib==ic),return(0)); \\ self intersection
  if(ia==ic,return(+intersection_abad(A,ia,ib,ic,id)));
  if(ib==ic,return(+intersection_abbd(A,ia,ib,ic,id)));
  if(ia==id,return(-intersection_abbd(A,ic,id,ia,ib)));
  if(ib==id,return(+intersection_abcb(A,ia,ib,ic,id)));
  \\ transverse case, cannot occur for a maximum tree
  return(intersection_inner(A,ia,ib,ic,id));
};


intersection_abbd(A,ia,ib,ic,id) = {
  my(k,a,b,d,tau,Aprim,fab,endarg,arg1,arg2,fbd);
  k = #A;
  a = A[ia]; b = A[ib]; d = A[id];
  tau = arg((d-b)/(b-a));
  /* \frac{\sqrt{d-b}^{k}\prod_{λ\neq b,d}\sqrt{x_β(b)-ψ_β(λ)}} */
  \\fbd = sqrt((d-b))^k
  \\*prod(i=1,k,if(i==ib||i==id,1,sqrt(-1-(2*A[i]-b-d)/(d-b))));
  Aprim = make_Aprim(A,ib,id);
  fbd = sqrt((d-b))^k*sqrt_affinereduction(Aprim,-1);
  \\fab = sqrt((b-a))^k
  \\*prod(i=1,k,if(i==ib||i==ia,1,sqrt(+1-(2*A[i]-b-a)/(b-a))));
  Aprim = make_Aprim(A,ia,ib);
  fab = sqrt((b-a))^k*sqrt_affinereduction(Aprim,1);
  if(default(debug)>=1,
      endarg = arg(fab/fbd);
      arg1 = -(Pi+tau)/2;
      arg2 = (Pi-tau)/2;
      if(abs(endarg-arg1)>0.0001&&abs(endarg-arg2)>0.0001,
        print("intersection_abbd");
        print(" tau=",tau);
        printf(" arg[fab/fbd]=%1.4f should be %1.4f or %1.4f\n",endarg,arg1, arg2);
        printf("tau/2=%1.3f ; 2*tau=%1.3f ; (Pi-tau)/2=%1.3f ; Pi-2*tau=%1.3f ; \n",
          tau/2,2*tau,(Pi-tau)/2,Pi-2*tau);
        );
    );
  if(imag(fbd/fab)<0,return(-1),return(1));
};
intersection_abcb(A,ia,ib,ic,id) = {
  my(k,a,b,c,tau,Aprim,fab,fcb,endarg,arg1,arg2);
  k = #A;
  a = A[ia]; b = A[ib]; c = A[ic];
  tau = arg((b-c)/(b-a));
  \\ \frac{\sqrt{d-b}^{k}\prod_{λ\neq b,d}\sqrt{x_β(b)-ψ_β(λ)}}
  \\fcb = sqrt((b-c))^k
    \\*prod(i=1,k,if(i==ic||i==ib,1,sqrt(+1-(2*A[i]-c-b)/(b-c))));
  Aprim = make_Aprim(A,ic,ib);
  fcb = sqrt((b-c))^k * sqrt_affinereduction(Aprim,+1);
  \\fab = sqrt((b-a))^k
    \\*prod(i=1,k,if(i==ib||i==ia,1,sqrt(+1-(2*A[i]-b-a)/(b-a))));
  Aprim = make_Aprim(A,ia,ib);
  fab = sqrt((b-a))^k*sqrt_affinereduction(Aprim,+1);
  if(default(debug)>=1,
      endarg = arg(fab/fcb);
      \\arg1 = tau/2;
      arg1 = if(tau>0,Pi-tau/2,-Pi-tau/2);
      \\arg2 = Pi-tau/2;
      arg2 = -tau/2;
      if(abs(endarg-arg1)>0.0001&&abs(endarg-arg2)>0.0001,
        print("intersection_abcb");
        print(" tau=",tau);
        printf(" arg[fab/fcb]=%1.4f should be %1.4f or %1.4f\n",endarg,arg1, arg2);
        printf("tau/2=%1.3f ; 2*tau=%1.3f ; (Pi-tau)/2=%1.3f ; Pi-2*tau=%1.3f ; Pi-tau/2=%1.3f ; Pi+tau/2=%1.3f \n", tau/2,2*tau,(Pi-tau)/2,Pi-2*tau,Pi-tau/2,Pi+tau/2);
        );
    );
  if(imag(fab/fcb)<0,return(-1),return(1));
};
intersection_abad(A,ia,ib,ic,id) = {
  my(k,a,b,d,tau,Aprim,fab,endarg,arg1,arg2,fad,j);
  k = #A;
  a = A[ia]; b = A[ib]; d = A[id];
  tau = arg((d-a)/(b-a));
  \\ \frac{\sqrt{d-b}^{k}\prod_{λ\neq b,d}\sqrt{x_β(b)-ψ_β(λ)}}
  \\fad = sqrt((d-a))^k
    \\*prod(i=1,k,if(i==ia||i==id,1,sqrt(-1-(2*A[i]-a-d)/(d-a))));
  Aprim = make_Aprim(A,ia,id);
  fad = sqrt((d-a))^k*sqrt_affinereduction(Aprim,-1);
  \\fab = sqrt((b-a))^k
    \\*prod(i=1,k,if(i==ib||i==ia,1,sqrt(-1-(2*A[i]-b-a)/(b-a))));
  Aprim = make_Aprim(A,ia,ib);
  j=0;for(i=1,k,if(i==ia||i==ib,,Aprim[j++]=(2*A[i]-a-b)/(b-a)));
  fab = sqrt((b-a))^k*sqrt_affinereduction(Aprim,-1);
  if(default(debug)>=1,
      endarg = arg(fab/fad);
      arg1 = -tau/2;
      arg2 = if(tau>0,Pi-tau/2,-Pi-tau/2);
      if(abs(endarg-arg1)>0.0001&&abs(endarg-arg2)>0.0001,
        print("intersection_abad");
        print(" tau=",tau);
        printf(" arg[fab/fad]=%1.4f should be %1.4f or %1.4f\n",endarg,arg1, arg2);
        printf("tau/2=%1.3f ; 2*tau=%1.3f ; (Pi-tau)/2=%1.3f ; Pi-2*tau=%1.3f ; Pi-tau/2=%1.3f ; Pi+tau/2=%1.3f \n", tau/2,2*tau,(Pi-tau)/2,Pi-2*tau,Pi-tau/2,Pi+tau/2);
        );
    );
  if(imag(fab/fad)<0,return(-1),return(1));
};


/* inner intersection I[ab].I[cd] */
/* assume different end points */
intersection_inner(A,ia,ib,ic,id) = {
  my(a,b,c,d,cprim,dprim,imc,imd,xp,fp,xpab,p,xpcd,fpab,fpcd,aprim,bprim,xpcd2,tpab,tpcd,refab,refp);
\\  k = #A;
  a = A[ia]; b = A[ib]; c = A[ic]; d = A[id];
  /* first rule out null intersection */
  cprim = (2*c-a-b)/(b-a);
  dprim = (2*d-a-b)/(b-a);
  imc = imag(cprim);
  imd = imag(dprim);
  if(sign(imc)*sign(imd)==1,return(0)); \\ on the same side
    /* p the intersection */
    xp = imag(conj(cprim)*(dprim-cprim));
  fp = imd-imc;
  if(abs(xp)>=abs(fp),return(0)); \\ discard if xp not in ]-1,1[
    xpab = xp/fp;
  \\ compare the sheets at the intersection p */
    p = (b-a)/2*xpab+(b+a)/2;
  xpcd = (2*p-c-d)/(d-c); \\ should be in ]-1,1[
    /* value 1/y(p) on [ab] */
    fpab = sqrt(b-a)^#A * prod(i=1,#A,if(i==ia||i==ib,1,sqrt(xpab-(2*A[i]-a-b)/(b-a))));
  /* value 1/y(p) on [cd] */
  fpcd = sqrt(d-c)^#A * prod(i=1,#A,if(i==ic||i==id,1,sqrt(xpcd-(2*A[i]-c-d)/(d-c))));
  if(default(debug)>2,
      printf("intersection_inner "
        "[%1.2f%+1.2f*I,%1.2f%+1.2f*I].[%1.2f%+1.2f*I,%1.2f%+1.2f*I]\n",
      real(a),imag(a),real(b),imag(b),real(c),imag(c),real(d),imag(d));
      \\ check everything is correct
      /* xpcd */
      aprim = (2*a-c-d)/(d-c);
      bprim = (2*b-c-d)/(d-c);
      xpcd2 = imag(conj(aprim)*(bprim-aprim))/imag(bprim-aprim);
      print(" xpcd=",xpcd,"=",xpcd2);
      /* quotient and cosh quotient */
      tpab = atanh(xpab);
      tpcd = atanh(xpcd);
      print(" fpab/fpcd=",fpab/fpcd);
      print(" cosh[tpab]/cosh[tpcd]=",cosh(tpab)/cosh(tpcd));
      refab = I*cosh(tpab)/fpab*2^(#A/2); refp = 1/prod(i=1,#A,sqrt(p-A[i]));
      printf(" and fpab=%1.3f%+1.3fI should be +/-f[p]=%1.3f%+1.3fI\n",
        real(refab),imag(refab),real(refp),imag(refp));
    );
  \\ result : sign(fp) gives intersection number [ab].[cd]
    \\ if sgn(fpab/fpcd)=+1
    return(2*sign(fp)*sign(real(fpab/fpcd)));
};


make_Aprim(A,ia,ib=0) = {
my(p,a,b,j,Aprim);
p=0;
a=A[ia];
j=0;
if(ib, 
    Aprim=vector(#A-2);
    b=A[ib];
    for(i=1,#A,if(i!=ia&&i!=ib,Aprim[j++]=(2*A[i]-a-b)/(b-a)));
   , 
    Aprim=vector(#A-1);
    for(i=1,#A,if(i!=ia,Aprim[j++]=(2*A[i]-a-p)/(p-a)));
    );
return(Aprim);
};


/*
make_Aprim(A,ia,ib=0) = 
{my(p);p=0;
  if(ib, 
    my(Aprim=vector(#A-2));
    my(a = A[ia]); my(b=A[ib]); my(j=0);
    for(i=1,#A,i==ia||i==ib||Aprim[j++]=(2*A[i]-a-b)/(b-a));
    return(Aprim);
   , 
    my(Aprim=vector(#A-1));
    my(a=A[ia]);my(j=0);
    for(i=1,#A,i==ia||Aprim[j++]=(2*A[i]-a-p)/(p-a));
    return(Aprim);
    );
};
*/

sqrt_affinereduction(Aprim,z,product=0) = {
  my(z1,i1,z2,i2,sgn);
  product=1; 
    if(product||#Aprim==1,
        return(prod(i=1,#Aprim,
            if(real(Aprim[i])>0,I*sqrt(Aprim[i]-z),sqrt(z-Aprim[i]))
            ))
      );
    sgn = 1;
  z1 = z-Aprim[1];
  if(imag(z1)==0&&real(z1)<0,z1=-z1;sgn*=-I);
  i1 = isign(z1);
  for(i=2,#Aprim,
      z2 = z-Aprim[i];
      if(imag(z2)==0&&real(z2)<0,z2=-z2;sgn *=-I);
      i2 = isign(z2);
      z1 *= z2;
      if(i1==i2,
        i1 = isign(z1);
        if(i1!=i2,sgn*=-1),
        i1 = isign(z1)
        );
     );
  return(sqrt(z1)*sgn);
};


isign(x) = { my(s);s=sign(imag(x));if(s,s,1);};



periods_spanning(C) = {
\\  my(g = C[1]);
  my(tree = C[3]);
  my(A = C[2]);
  my(keependpts,res,tmp,endpointsvalues);
  if(keependpts=1,
  Mat(vector(#tree,k,2*int_periods_affinereduction(C,tree[k][1],tree[k][2])[1]~));
  ,
  res = vector(#tree);
  endpointsvalues=vector(#tree);
  for(k=1,#tree,
    tmp = int_periods_affinereduction(A,tree[k][1],tree[k][2]);
    res[k] = 2*tmp[1]~;
    endpointsvalues[k] = [tmp[2],tmp[3]];
  );
  Mat(res);
  );
};



int_periods_affinereduction(C,i1,i2,decomp=0) = {
  my(A,a,b,int_points,geom_factor,decprim,Aprim,res,x,dx,tp,tm,fact,vplus,vmoins); 
  A = C[2];
  int_points = C[15];
  a = A[i1]; b = A[i2];
  if(real(a)>real(b),print("bad order of edge"));
\\  dec = (a+b)/2;
  geom_factor = (b-a)/2;
  decprim = (b+a)/(b-a);
  Aprim = make_Aprim(A,i1,i2);
  my(g=C[1]);
  res = vector(g);
  res[1] = 1/sqrt_affinereduction(Aprim,0,decomp);
  for(j=1,g-1,res[j+1] = res[j] * decprim);
  for(i=1,#int_points,
      x = int_points[i][1];
      dx = int_points[i][2];
      tp = dx/sqrt_affinereduction(Aprim,+x,decomp);
      tm = dx/sqrt_affinereduction(Aprim,-x,decomp);
        res[1] += (tp+tm);
      for(j=2,g,
        tp *= (decprim+x);
        tm *= (decprim-x);
        res[j] += (tp+tm);
      );
     );
   fact = I*C[14]/sqrt(geom_factor)^#Aprim;
   res[1] *= fact;
   for(j=2,g,
     fact *= geom_factor;
     res[j] *= fact;
     );
   vplus = sqrt_affinereduction(Aprim,1,decomp)*sqrt(geom_factor)^#Aprim/geom_factor;
   vmoins = sqrt_affinereduction(Aprim,-1,decomp)*sqrt(geom_factor)^#Aprim/geom_factor;
   return([res,vmoins,vplus]);
};



integration_parameters(A,tree,tau,D,provenbounds=0) = {
my(M1,M2,i1,i2,h,npoints);
  if(provenbounds,
    tau *= .95; \\ do it precisely
    M1 = M2 = 0;
    for(i=1,#tree,
      i1 = tree[i][1]; i2 = tree[i][2];
      my(Aprim = make_Aprim(A,i1,i2));
      M1 = max(M1,bound_M1(Aprim));
      M2 = max(M2,bound_M2(Aprim,tau));
      );
    ,
    M1 = M2 = 1; \\ not true
    );
  h = h_opt(D,tau,M2);
  npoints = ceil(t_opt(D,M1)/h);
  return([h,npoints]);
};


integration_points_thsh(h,npoints,lambda=Pi/2) = {
my(eh,eh_inv,res,ekh,ekh_inv,sh,thsh,ch2,esh,esh_inv,chsh2_i,shsh2); 
eh = exp(h);
eh_inv = 1/eh;
ekh = ekh_inv = 1;
res = vector(npoints);
for(k=1,npoints,
ekh *= eh;
ekh_inv *= eh_inv;
      sh = (ekh-ekh_inv)/2;
      ch2 = (ekh+ekh_inv); /* 2*cosh(kh) */
      esh = exp(lambda*sh);
      esh_inv = 1/esh;
      chsh2_i = 1/(esh+esh_inv); \\ 1/(2*cosh(lambda*sinh(kh)))
      shsh2 = esh-esh_inv; \\ 2*sinh(lambda*sinh(kh))
      thsh = shsh2*chsh2_i;
      res[k] = [thsh,ch2*chsh2_i];
     );
  return([h*lambda,res]);
};

bound_M1(Aprim) = {
  1/sqrt(prod(k=1,#Aprim,dist_1(Aprim[k])));
};

bound_M2(Aprim,tau) = {
  1/sqrt(prod(k=1,#Aprim,dist_thsh(Aprim[k],tau)));
};

t_opt(D,M1,lambda=Pi/2)={asinh((D+log(4*M1))/lambda)};

h_opt(D,tau,M2,lambda=Pi/2)={2*Pi*tau/log(1+2*M2*phi_bound(tau,lambda,0.5)*exp(D))};

dist_1(p) = {
  my(xp = abs(real(p)));
  if(xp>1,abs(xp-1+I*imag(p)),abs(imag(p)));
};

phi(x,tau,lambda)={tanh(lambda*sinh(x+I*tau))};
argphip(x,tau,lambda) = {arg(cosh(x+I*tau))-2*arg(cosh(lambda*sinh(x+I*tau)))};
argphi(x,p,tau,lambda)={arg(p-phi(x,tau,lambda))};


dist_thsh(p,tau,lambda=Pi/2) = {
my(x0,x1,x);             
  p = abs(real(p))+I*abs(imag(p));
  x0 = 0; x1 = acosh(Pi/(2*lambda*sin(tau)));
  x = solve(x=x0,x1, argphi(x,p,tau,lambda)-argphip(x,tau,lambda)-Pi/2);
  abs(p-phi(x,tau,lambda));
};


phi_bound(tau,lambda=Pi/2,alpha=1/2) = {
my(Ytau,Xtau);
  Ytau = sqrt(Pi/2*lambda*sin(tau));
  Xtau = sqrt(Ytau^2-lambda^2*sin(tau)^2)/tan(tau);
  return((1+2*alpha)/(2*alpha*cos(Ytau)^(2*alpha))+1/(2*alpha*sinh(Xtau)^(2*alpha)));
};

max_spanning(A,nedges=0)= {
  my(n,tau_v,z,taken,tree,taumin,i,j);
  n = #A;
  if(!nedges,nedges=n-1);
  tau_v=vector(n*(n-1)/2);
  z=0;
  for(i=1,n,
    for(j=i+1,n,
      tau_v[z++] = [tau_edge(A,i,j),i,j];
      if(real(A[i])>real(A[j]),tau_v[z][2]=j;tau_v[z][3]=i);
      );
    );
  tau_v = vecsort(tau_v,1); \\ sort according to tau
  taken = vector(n,i,0); \\ mark vertex if in tree
  tree = vector(nedges);
  z=n*(n-1)/2; \\select edge with maximum tau
  taken[tau_v[z][2]]=1; \\ z taken at first loop
  taumin = tau_v[z][1];
  for(k=1,nedges,
    z=n*(n-1)/2;
    while(taken[tau_v[z][2]]+taken[tau_v[z][3]]!=1,z--);
    i = tau_v[z][2]; j = tau_v[z][3];
    tree[k] = [i,j];
    taumin = min(taumin,tau_v[z][1]);
    taken[i] = taken[j] = 1;
    );
  return([tree,taumin]);
};

tau_edge(A,i,j,lambda=Pi/2) = {
my(tauij);   
 tauij = 4; 
  for(k=1,#A,
    if(k==i||k==j,next());
      tauij = min(tauij,tau_3(A[i],A[j],A[k]));
      );
   return(tauij);
};

tau_3(ai,aj,ak,lambda=Pi/2,verbose=0) = {
  my(zk,xItau);
  zk = (2*ak-ai-aj)/(aj-ai);
  xItau = asinh(atanh(zk)/lambda);
  return(abs(imag(xItau)));
};

hcInit(A,provenbounds=0) = {
  my(hcStruct = vector(19));
  my(Dcalc,g,tree,tau,h,npoints,IntFactor,IntPoints,coh1x_homC,coh1x_homAB,IntC,ABtoC);
  my(Omega0,Omega1);


  g =2;
  hcStruct[2] = A;
  hcStruct[1] = g;
  hcStruct[13] = Dcalc = precision(1.);
  /* low precision parameters */
  default(realprecision,19);


  my(tmp = max_spanning(A,2*g));
  hcStruct[3] = tree = tmp[1];
  hcStruct[12] = tau = tmp[2];
  my(D = Dcalc*log(10));
  tmp = integration_parameters(A,tree,tau,D,provenbounds);
  hcStruct[11] = h = precision(tmp[1],Dcalc);
  hcStruct[10] = npoints = tmp[2];
  default(realprecision,Dcalc);
  tmp = integration_points_thsh(h,npoints); \\ bad...

  IntFactor = tmp[1];
  IntPoints = tmp[2];
  hcStruct[14] = IntFactor;
  hcStruct[15] = IntPoints;

\\print("intpoints="#IntPoints);

  coh1x_homC = periods_spanning(hcStruct);
  IntC = intersection_spanning(A,tree);
  ABtoC = symplectic_reduction(IntC);
  coh1x_homAB = coh1x_homC * ABtoC; /* now matrix in (A,B) basis */
  Omega0 = vecextract(coh1x_homAB,Str("1..",g),Str("1..",g));
  Omega1 = vecextract(coh1x_homAB,Str("1..",g),Str(g+1,".."));

  return(Omega1^(-1)*Omega0);
};




/************* FIN Matrice des périodes ***********/
/**************************************************/