#include <stdio.h>
#include <mpc.h>
#include <pari/pari.h>
#include <cmh.h>

/*
GP; install ("mul", "GGp", , "./mpfrc-pari.so");
*/

static void xmpn_mirrorcopy(mp_limb_t *z, mp_limb_t *x, long n)
{
  long i;
  for(i=0;i<n;i++)
    z[i]=x[n-1-i];
}


void mpfr_init_set_real (mpfr_ptr x, GEN r)
   /* creates space for x on the pari stack and sets it to r, which is
      supposed to be a t_REAL */
{
   mp_prec_t prec = realprec (r) - 2;
   GEN rc = new_chunk (prec);
   xmpn_mirrorcopy (rc, r+2, prec);
   mpfr_custom_init_set (x,
                         signe (r) ? MPFR_REGULAR_KIND : MPFR_ZERO_KIND,
                         expo (r) + 1,
                         BITS_IN_LONG * prec,
                         rc);
   if (signe(r)<0) mpfr_neg(x,x,MPFR_RNDN);

   /*   pari_printf("mpfrinit: %Ps (%d)",r,signe(r)* MPFR_REGULAR_KIND);
   printf("\n");
   mpfr_out_str (stdout, 10, 0, x, MPFR_RNDN);
   printf("\n");
   */
   

}


void mpfr_init_set_GEN (mpfr_ptr x, GEN r, long prec)
   /* creates space for x on the pari stack and sets it to r.
      If r is a t_REAL, its own precision is used;
      if r is a t_INT, prec is used as pari precision. */
{
  switch (typ (r)) {
  case t_REAL:
    mpfr_init_set_real (x, r);
    break;
  case t_INT:
    mpfr_init_set_real (x, itor (r, prec));
    break;
  default:
    pari_err_TYPE ("mpfr_init_set_GEN", r);
  }
}


void mpc_init_set_GEN (mpc_ptr z, GEN c, long prec)
   /* creates space for z on the pari stack and sets it to c.
      prec is passed through to mpfr_init_set_GEN. */
{
   if (typ (c) != t_COMPLEX) {
      mpfr_init_set_GEN (mpc_realref (z), c, prec);
      mpfr_init_set_GEN (mpc_imagref (z), gen_0, prec);
   }
   else {
      mpfr_init_set_GEN (mpc_realref (z), gel (c, 1), prec);
      mpfr_init_set_GEN (mpc_imagref (z), gel (c, 2), prec);
   }
}



void
mpfr_print (GEN r, long prec)
{
   pari_sp av = avma;
   mpfr_t x;

   mpfr_init_set_GEN (x, r, prec);
   mpfr_out_str (stdout, 10, 0, x, MPFR_RNDN);
   printf ("\n");

   avma = av;
}


void mpc_print (GEN c, long prec)
{
   pari_sp av = avma;
   mpc_t z;

   mpc_init_set_GEN (z, c, prec);
   mpc_out_str (stdout, 10, 0, z, MPC_RNDNN);
   printf ("\n");

   avma = av;
}



void mpfr_pari_init (mpfr_ptr x, long prec)
   /* initialises x on the pari stack with the pari precision prec */
{
   mpfr_init_set_GEN (x, gen_0, prec);
}

void mpc_pari_init (mpc_ptr z, long prec)
   /* initialises z on the pari stack with the pari precision prec */
{
   mpc_init_set_GEN (z, gen_I (), prec);
}


GEN mpfr_get_GEN (mpfr_srcptr x)
   /* returns x as a t_REAL */
{
   long prec = mpfr_get_prec (x) / BITS_IN_LONG + 2;
   if (mpfr_zero_p (x))
      return real_0 (prec);
   else {
      GEN r = cgetr (prec);
      xmpn_mirrorcopy (r+2, mpfr_custom_get_significand (x), prec - 2);
      setsigne (r, (mpfr_signbit (x) ? -1 : 1));
      setexpo (r, mpfr_get_exp (x) - 1);
      return r;
   }
}


GEN mpc_get_GEN (mpc_srcptr z)
   /* returns z as a t_COMPLEX */
{
   GEN c = cgetg (3, t_COMPLEX);
   gel (c, 1) = mpfr_get_GEN (mpc_realref (z));
   gel (c, 2) = mpfr_get_GEN (mpc_imagref (z));
   return c;
}


GEN mul (GEN c1, GEN c2, long prec)
   /* test function */
{
   if (typ (c1) != t_COMPLEX && typ (c2) != t_COMPLEX) {
      mpfr_t z, z1, z2;
      mpfr_pari_init (z, prec);
      mpfr_init_set_GEN (z1, c1, prec);
      mpfr_init_set_GEN (z2, c2, prec);
      mpfr_mul (z, z1, z2, MPC_RNDNN);
      return mpfr_get_GEN (z);
   }
   else {
      mpc_t z, z1, z2;
      mpc_pari_init (z, prec);
      mpc_init_set_GEN (z1, c1, prec);
      mpc_init_set_GEN (z2, c2, prec);
      mpc_mul (z, z1, z2, MPC_RNDNN);
      return mpc_get_GEN (z);
   }
}



void get_taumpc_from_taugen (mpc_t taumpc[3],GEN taugen,long prec)
{
  mpc_init_set_GEN (taumpc[0], gcoeff(taugen,1,1), prec);  
  mpc_init_set_GEN (taumpc[1], gcoeff(taugen,2,1), prec);  
  mpc_init_set_GEN (taumpc[2], gcoeff(taugen,2,2), prec);
} 


GEN eval10theta2 (GEN taugen,long prec)
{
  mpc_t taumpc[3];
  GEN vec;
  int i;

  get_taumpc_from_taugen(taumpc,taugen,prec);

  mpc_t th2[10];
  for(i=0;i<10;i++)
    {mpc_pari_init(th2[i],prec);}

  eval_10theta2_naive(th2,taumpc);
  // eval_10theta2_newton (th2,taumpc,prec,0);

  vec=cgetg(11,t_VEC);
  for(i=0;i<10;i++)
    {gel(vec,i+1)=mpc_get_GEN(th2[i]);}
  
  return(vec);
}


GEN eval10theta2Newton (GEN taugen,long prec)
{
  mpc_t taumpc[3];
  GEN vec;
  int i;

  get_taumpc_from_taugen(taumpc,taugen,prec);

  mpc_t th2[10];
  for(i=0;i<10;i++)
    {mpc_pari_init(th2[i],prec);}

  eval_10theta2_newton (th2,taumpc,prec,0);

  vec=cgetg(11,t_VEC);
  for(i=0;i<10;i++)
    {gel(vec,i+1)=mpc_get_GEN(th2[i]);}
  
  return(vec);
}

GEN eval3bj (GEN taugen,long prec)
{
  mpc_t taumpc[3];
  GEN vec;
  int i;

  get_taumpc_from_taugen(taumpc,taugen,prec);

  mpc_t th2[3];
  for(i=0;i<3;i++)
    {mpc_pari_init(th2[i],prec);}

  eval_3theta2q_naive(th2,taumpc);
  
  vec=cgetg(4,t_VEC);
  for(i=0;i<3;i++)
    {gel(vec,i+1)=mpc_get_GEN(th2[i]);}
  
  return(vec);
}

GEN eval4theta (GEN taugen, long prec)
{
  mpc_t taumpc[3];
  GEN vec;
  int i;

  get_taumpc_from_taugen(taumpc,taugen,prec);

  mpc_t th2[4];
  for(i=0;i<4;i++)
    {mpc_pari_init(th2[i],prec);}

  //eval_4theta_naive_orig(th2,taumpc);
  eval_4theta_naive(th2,taumpc);
    
  vec=cgetg(5,t_VEC);
  for(i=0;i<4;i++)
    {gel(vec,i+1)=mpc_get_GEN(th2[i]);}
  
  return(vec);
}



GEN borchardt(GEN vec, long prec)
{
  mpc_t vecmpc[3];
  mpc_t r;
  int i;
  
  for(i=0;i<3;i++)
    {mpc_init_set_GEN (vecmpc[i], gel(vec,i+1), prec);}
  
  mpc_pari_init(r,prec);

  BorchardtMean3(r,vecmpc);

  return mpc_get_GEN (r);
}


 
GEN h4h10h12h16FromTh2 (GEN th2gen, long prec)
{
  mpc_t th2mpc[10];
  int i;
  mpc_t hmpc[4];  
  GEN hgen;

  for(i=0;i<10;i++)
    {mpc_init_set_GEN (th2mpc[i], gel(th2gen,i+1), prec);}

  for(i=0;i<4;i++)
    {mpc_pari_init(hmpc[i],prec);}

  h4h10h12h16_from_th2(hmpc,th2mpc);
  
  hgen=cgetg(5,t_VEC);
  for(i=0;i<4;i++)
    {gel(hgen,i+1)=mpc_get_GEN(hmpc[i]);}


  return (hgen);
}


GEN I2I4I6I10Fromh4h10h12h16 (GEN hgen, long prec)
{ 
  int i;
  mpc_t hmpc[4];
  mpc_t Impc[4];
  GEN Igen;

  for(i=0;i<4;i++)
    {mpc_init_set_GEN (hmpc[i], gel(hgen,i+1), prec);}

  for(i=0;i<4;i++)
    {mpc_pari_init(Impc[i],prec);}
  
   I2I4I6I10_from_h4h10h12h16(Impc, hmpc);
  
  Igen=cgetg(5,t_VEC);
  for(i=0;i<4;i++)
    {gel(Igen,i+1)=mpc_get_GEN(Impc[i]);}

  return(Igen);


}

GEN StrengInvFromI2I4I6I10 (GEN Igen, long prec)
{
 int i;
  mpc_t Impc[4];
  mpc_t Jinvmpc[3];
  GEN Jinvgen;

  for(i=0;i<4;i++)
    {mpc_init_set_GEN (Impc[i], gel(Igen,i+1), prec);}

  for(i=0;i<4;i++)
    {mpc_pari_init(Jinvmpc[i],prec);}
  
   streng_i1i2i3_from_I2I4I6I10(Jinvmpc, Impc);
  
  Jinvgen=cgetg(4,t_VEC);
  for(i=0;i<3;i++)
    {gel(Jinvgen,i+1)=mpc_get_GEN(Jinvmpc[i]);}

  return(Jinvgen);
}



  

  
